/*
 * IPipe.java
 *
 * Created on December 9, 2004, 10:18 AM
 */

package ipipe;

import java.io.*;
import java.net.*;

import javax.net.ssl.*;

/**
 *
 * @author  jRobertson
 */
public class IPipe {
    
    int listenPort;
    int destPort;
    String destHost;
    
    OutputStream clientLogStream;
    OutputStream serverLogStream;
    OutputStream errorStream;
    
    /** Creates a new instance of IPipe */
    public IPipe(int l, String h, int p, OutputStream cls, OutputStream sls, OutputStream errs) {
        this.listenPort = l;
        this.destHost = h;
        this.destPort = p;
        serverLogStream = sls;
        clientLogStream = cls;
        errorStream = errs;
    }
    
    private Runnable runWhenListening;

    /**
     * Holds value of property clientSSL.
     */
    private boolean clientSSL;

    /**
     * Holds value of property serverSSL.
     */
    private boolean serverSSL;

    /**
     * Holds value of property clientBuffer.
     */
    private int clientBuffer;

    /**
     * Holds value of property serverBuffer.
     */
    private int serverBuffer;
    
    public void setRunWhenListening(Runnable r) {
        this.runWhenListening = r;
    }
    
    public void listen() throws java.io.IOException {
        ServerSocket ss;
        
        if(clientSSL) {
            ss = SSLServerSocketFactory.getDefault().createServerSocket(listenPort);
        } else {
            ss = new ServerSocket(listenPort);
        }
        
        new Thread(runWhenListening).start();
        
        while(true) {

            Socket ins = ss.accept();
            
            Socket outs;
            
            if( serverSSL ) {
                outs =
                SSLSocketFactory.getDefault().createSocket(destHost, destPort);
            } else {
               outs = new Socket(destHost, destPort);
            }
            
            InputStream fromClient = ins.getInputStream();
            InputStream fromServer = outs.getInputStream();
            OutputStream toClient = ins.getOutputStream();
            OutputStream toServer = outs.getOutputStream();
            
            if(serverBuffer != 0) {
                fromServer = new BufferedInputStream(fromServer,serverBuffer);
                toClient = new BufferedOutputStream(toClient,serverBuffer);
            }
            
            if(clientBuffer != 0){
                fromClient = new BufferedInputStream(fromClient,clientBuffer);
                toServer = new BufferedOutputStream(toClient,clientBuffer);
            }
            
            
            new Thread(new Copier(fromClient,toServer,clientLogStream,ins,outs)).start();
            new Thread(new Copier(fromServer,toClient,serverLogStream,ins,outs)).start();
            
        }
        
    }

    /**
     * Getter for property clientSSL.
     * @return Value of property clientSSL.
     */
    public boolean isClientSSL() {

        return this.clientSSL;
    }

    /**
     * Setter for property clientSSL.
     * @param clientSSL New value of property clientSSL.
     */
    public void setClientSSL(boolean clientSSL) {

        this.clientSSL = clientSSL;
    }

    /**
     * Getter for property serverSSL.
     * @return Value of property serverSSL.
     */
    public boolean isServerSSL() {

        return this.serverSSL;
    }

    /**
     * Setter for property serverSSL.
     * @param serverSSL New value of property serverSSL.
     */
    public void setServerSSL(boolean serverSSL) {

        this.serverSSL = serverSSL;
    }

    /**
     * Getter for property clientBuffer.
     * @return Value of property clientBuffer.
     */
    public int getClientBuffer() {

        return this.clientBuffer;
    }

    /**
     * Setter for property clientBuffer.
     * @param clientBuffer New value of property clientBuffer.
     */
    public void setClientBuffer(int clientBuffer) {

        this.clientBuffer = clientBuffer;
    }

    /**
     * Getter for property serverBuffer.
     * @return Value of property serverBuffer.
     */
    public int getServerBuffer() {

        return this.serverBuffer;
    }

    /**
     * Setter for property serverBuffer.
     * @param serverBuffer New value of property serverBuffer.
     */
    public void setServerBuffer(int serverBuffer) {

        this.serverBuffer = serverBuffer;
    }
    
    class Copier implements Runnable {
    
        private InputStream source;
        private OutputStream sink;
        private OutputStream logStream;
        private Socket s1, s2;
        
        public Copier(InputStream is, OutputStream os, OutputStream log, Socket sa, Socket sb) {
            this.source = is;
            this.sink = os;
            this.s1 = sa;
            this.s2 = sb;
            this.logStream = log;
        }
        
        public void run() {
            
            int c;

            try {
                while( (c=source.read()) != -1) {
                    sink.write(c);
                    if(logStream != null) {
                        logStream.write(c);
                    }
                }
            } catch (SocketException e) {
                // normal since the socket might have closed
            } catch (Exception e) {
                if(errorStream != null)
                    e.printStackTrace(new PrintStream(errorStream));
                    
            }

            try {
                source.close();
            } catch(Exception e) {
            }
            try {
                sink.close();
            } catch(Exception e) {
            }
            try {
                s1.close();
            } catch(Exception e) {
            }
            try {
                s2.close();
            } catch(Exception e) {
            }
            try {
                logStream.flush();
            } catch(Exception e) {
            }
                
        }
        
    }    
        
}
